extern crate grep;

use std::env;
use std::fs::File;
use std::io::prelude::*;
use std::process;
use std::error::Error;

struct Config {
    query: String,
    filename: String,
}

fn main() {
    let args: Vec<String> = env::args().collect();

    let conf = Config::new(&args).unwrap_or_else(|err| {
        println!("Problem parsing arguments: {}", err);
        process::exit(1);
    });
    //let filename = &conf.filename.clone();

    if let Err(e) = run(conf) {
        println!("Big bad bada boom!\n{:?}", e);
        //println!("Big bad bada boom!\n{:?}", e.kind());
        //println!("Big bad bada boom!\n{}", e.description());

        process::exit(1);
    }
}

//fn run(conf: Config) -> Result<(), Box<std::io::Error>>
fn run(conf: Config) -> Result<(), Box<Error>> {
    println!("{:?} {:?}", conf.query, conf.filename);

    let mut f = File::open(&conf.filename)?;

    let mut contents = String::new();
    f.read_to_string(&mut contents)?;

    println!("Searching for '{}'", conf.query);
    println!("In file '{}'", conf.filename);
    println!("With text:\n{}", contents);

    Ok(())
}

impl Config {
    fn new(args: &[String]) -> Result<Config, &'static str> {
        if args.len() < 3 {
            return Err("too few arguments")
        }

        Ok(Config {
            query: args[1].clone(),
            filename: args[2].clone(),
        })
    }
}
